#include <ecf/ECF.h>
#include <algorithm>
#include "genotip.h"
#include "operatori.h"
#include "gui.h"

IMPLEMENT_APP(Mozart)

MainWindow::MainWindow(const wxString &title) 
	: wxFrame(NULL, wxID_ANY, title, wxDefaultPosition, wxSize(650, 250)) {
	
	menubar = new wxMenuBar;
	program = new wxMenu;
	izlaz   = new wxMenuItem(program, wxID_EXIT, _T("&Izlaz"));

	program->Append(izlaz);
	menubar->Append(program, _T("&Program"));
	SetMenuBar(menubar);

	CreateStatusBar();

	wxPanel *panel = new wxPanel(this, -1);
	wxBoxSizer *hboxMain = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *vboxNote = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer *vboxKonf = new wxBoxSizer(wxVERTICAL);
	
	wxBoxSizer *hboxGrid = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *hboxButtons = new wxBoxSizer(wxHORIZONTAL);
	wxGridSizer *gs = new wxGridSizer(3, 2, 3, 3); 

	wxStaticText *st1 = new wxStaticText(panel, wxID_ANY, 
			_T("Velicina populacije:"));
	txtVelPop = new wxTextCtrl(panel, wxID_ANY);
	wxStaticText *st2 = new wxStaticText(panel, wxID_ANY,
			_T("Broj generacija:"));
	txtNumIter = new wxTextCtrl(panel, wxID_ANY);
	wxStaticText *st3 = new wxStaticText(panel, wxID_ANY,
			_T("Vjerojatnost mutacije:"));
	txtMutProb = new wxTextCtrl(panel, wxID_ANY);

	gs->Add(st1, 0, wxEXPAND);
	gs->Add(txtVelPop, 0, wxEXPAND);
	gs->Add(st2, 0, wxEXPAND);
	gs->Add(txtNumIter, 0, wxEXPAND);
	gs->Add(st3, 0, wxEXPAND);
	gs->Add(txtMutProb, 0, wxEXPAND);

	btnRun = new wxButton(panel, ID_RUN, _T("Pokreni"));
	btnExport = new wxButton(panel, ID_EXPORT, _T("Spremi"));

	hboxGrid->Add(gs, 1, wxEXPAND);
	hboxButtons->Add(btnRun, 1);
	hboxButtons->Add(btnExport, 0, wxLEFT | wxBOTTOM, 5);

	crtovlje = new DrawingArea(panel);
	crtovlje->SetBackgroundColour(*wxWHITE);
	crtovlje->SetScrollbars(10, 10, 85, 10);
	crtovlje->Scroll(0, 0);

	vboxNote->Add(crtovlje, 1, wxEXPAND | wxALL, 10);

	vboxKonf->Add(hboxGrid, 0, wxEXPAND, 10);
	vboxKonf->Add(-1, 10);
	vboxKonf->Add(hboxButtons, 0, wxALIGN_RIGHT | wxRIGHT, 10);

	hboxMain->Add(vboxNote, 1, wxEXPAND | wxALL, 2);
	hboxMain->Add(vboxKonf, 0, wxEXPAND | wxTOP | wxRIGHT, 10);

	panel->SetSizer(hboxMain);

	Centre();
}

void MainWindow::OnQuit(wxCommandEvent &event) {
	Close(true);
}

void MainWindow::OnExport(wxCommandEvent &event) {
	if (crtovlje->tonovi.size() <= 0)
		return;

	wxFileDialog *saveDialog = new wxFileDialog(this, 
			_T("Odabir datoteke"), _T(""), _T(""), _T("*.ly"),
			wxFD_SAVE);
	if (saveDialog->ShowModal() != wxID_OK) 
		return;

	wxString filename = saveDialog->GetPath();
	wxTextFile lily;

	const wxString imena[] = {
	        _T("a"), _T("a#"), _T("b"), _T("c'"), _T("c#'"), 
		_T("d'"), _T("d#'"), _T("e'"), _T("f'"), _T("f#'"),
	       	_T("g'"), _T("g#'"), _T("a'"), _T("a#'"), _T("b'"), 
		_T("c''"), _T("c#''"), _T("d''"), _T("d#''"), _T("e''"), 
		_T("f''"), _T("f#''"), _T("g''")
	};

	if (lily.Create(filename) || lily.Open(filename)) {
		lily.Clear();

		lily.AddLine( _T("\\score {") );
		lily.AddLine( _T("{") );
		wxString out;

		for (int i = 0; i < (int)crtovlje->tonovi.size(); ++i)
		       out << imena[ (int)NoteVector::n[ (int)crtovlje->tonovi[i] ] ] << _T(" ");	

		lily.AddLine( out );

		lily.AddLine( _T("}") );
		lily.AddLine( _T("\\midi { }") );
		lily.AddLine( _T("\\layout { }") );
		lily.AddLine( _T("}") );

		lily.Write();
		lily.Close();
	}
}

void MainWindow::OnRun(wxCommandEvent &event) {
	uint *population_size = new uint(30);
	uint *maxgen = new uint(30);
	double *indprob = new double(0.3);

	if (! txtVelPop->GetValue().ToLong((long *)population_size))
		return;
	if (! txtNumIter->GetValue().ToLong((long *)maxgen))
		return;
	if (! txtMutProb->GetValue().ToDouble(indprob))
		return;

	StateP state  = static_cast<StateP> (new State);
	GenotypeP gen = static_cast<GenotypeP> (new NoteVector);

	state->getRegistry()->registerEntry("population.size", (voidP)population_size, UINT);
	state->getRegistry()->registerEntry("term.maxgen", (voidP)maxgen, UINT);
	state->getRegistry()->registerEntry("mutation.indprob", (voidP)indprob, DOUBLE);

	state->evalOp_ = static_cast<EvaluateOpP> (new NoteVectorEval);
	state->setGenotype(gen);

	char *argv[] = { NULL };
	state->initialize(0, argv);
	state->run();

	crtovlje->postaviNote(
		((NoteVector *) (state->best_->bestIndividual[0]->at(0).get()))
		->note );

	double fitness = state->best_->bestIndividual[0]->fitness->value_;
	wxString status;
       
	status.Printf( _T("Najbolji fitness: %d"), (int)fitness);

	SetStatusText( status );
	
	crtovlje->Refresh();
}

DrawingArea::DrawingArea(wxPanel *parent) : wxScrolledWindow(parent) {
	clef = new wxBitmap( _T("data/clef.png"), wxBITMAP_TYPE_PNG);
	note1 = new wxBitmap( _T("data/quarter_note_1.png"), wxBITMAP_TYPE_PNG);
	note2 = new wxBitmap( _T("data/quarter_note_2.png"), wxBITMAP_TYPE_PNG);
}

void DrawingArea::paintEvent(wxPaintEvent &event) {
	wxPaintDC dc(this);
	DoPrepareDC(dc);
	render(dc);
}

void DrawingArea::paint() {
	wxClientDC dc(this);
	DoPrepareDC(dc);
	render(dc);
}

void DrawingArea::render(wxDC &dc) {
	const int top = 50;
	const int e_a  = top+14;
	const int e_b  = top+42;

	for (int i = 0; i < 5; ++i)
		dc.DrawLine(10, top + i*12, 840, top + i*12);
	dc.DrawLine(10, top, 10, top + 4*12);
	for (int i = 0; i < 4; ++i)
		dc.DrawLine(240 + i*200, top, 240 + i*200, top + 4*12);

	dc.DrawBitmap(*clef, 15, top-12, true);
	for (int i = 0; i < min(16, (int)tonovi.size()); ++i) {
		int ton = tonovi[i];
		wxBitmap *note = ton < 9 ? note1 : note2;
		const int& e1 = ton < 9 ? e_a : e_b;

		int x = 60 + (i + i/4)*40;
		int y = e1 - (ton - 4)*6;

		// pomocne crte
		for (int j = 1; j <= (4-ton)/2; ++j)
			dc.DrawLine(x-4, top + (4+j)*12, x+20, top + (4+j)*12);
	
		for (int j = 1; j <= (ton-12)/2; ++j)
			dc.DrawLine(x-4, top - j*12, x+20, top - j*12);

		dc.DrawBitmap(*note, x, y, true);
	}
}


BEGIN_EVENT_TABLE(MainWindow, wxFrame)
EVT_MENU(wxID_EXIT, MainWindow::OnQuit)
EVT_BUTTON(ID_RUN, MainWindow::OnRun)
EVT_BUTTON(ID_EXPORT, MainWindow::OnExport)
END_EVENT_TABLE()

BEGIN_EVENT_TABLE(DrawingArea, wxScrolledWindow)
EVT_PAINT(DrawingArea::paintEvent)
END_EVENT_TABLE()

bool Mozart::OnInit() {
	wxInitAllImageHandlers();
	MainWindow *window = new MainWindow( _T("Mozart") );

	// defaultne vrijednosti parametara
	window->txtVelPop->SetValue( _T("30") );
	window->txtNumIter->SetValue( _T("30") );
	window->txtMutProb->SetValue( _T("0.3") );

	vector<char> tonovi;

	window->crtovlje->postaviNote(tonovi);

	window->Show(true);

	return true;
}
