#include <ecf/ECF.h>
#include "genotip.h"

bool NoteVectorMutation::mutate(GenotypeP gene) {
	NoteVector *nv = (NoteVector *) (gene.get());
	int idx, up;

	idx = state_->getRandomizer()->getRandomInteger(0, nv->duljina_ - 1);
	up  = state_->getRandomizer()->getRandomInteger(0, 1) == 0 ? -1 : 1;

	char &val = nv->note[idx];
	val = (val + up + NoteVector::upper_note) % NoteVector::upper_note;

	return true;
}

bool NoteVectorCrossover::mate(GenotypeP gen1, GenotypeP gen2, GenotypeP child) {
	NoteVector *p1 = (NoteVector *) (gen1.get());
	NoteVector *p2 = (NoteVector *) (gen2.get());
	NoteVector *ch = (NoteVector *) (child.get());

	uint pos = state_->getRandomizer()->getRandomInteger( 1, p1->duljina_ - 1);
	uint var = state_->getRandomizer()->getRandomInteger( 0, 1 );

	vector<char>& left  = (var == 0 ? p1->note : p2->note);
	vector<char>& right = (var == 0 ? p2->note : p1->note);

	for (uint i = 0; i < p1->duljina_; ++i) {
		vector<char>& cur = (i < pos ? left : right);
		ch->note[i] = cur[i];
	}

	return true;
}

FitnessP NoteVectorEval::evaluate(IndividualP individual) {
	FitnessP   fitness = static_cast<FitnessP> (new FitnessMax);
	NoteVector *nv     = (NoteVector *) individual->at(0).get();

	fitness->value_ = 0;
	for (uint i = 1; i < nv->duljina_; ++i) {
		int interval = NoteVector::n[ (int)nv->note[i] ] - NoteVector::n[ (int)nv->note[i-1] ];

		if (interval < 0)
			interval = -interval;

		int sc = score[ interval % 12 ];

		if (interval > 12)
			sc /= 5;

		fitness->value_ += sc;
	}

	return fitness;
}

/*
 * score predstavlja 'dobrotu' pojedinog intervala
 * nabacano otprilike
 *
 * prima        - ok (10)
 * mala sekunda - nije ok, ali ne zelimo izbjeci polutonski prijelaz h->c (5)? hm...
 * sekunda      - recimo ok (15)
 * mala terca   - jupi jej (50)
 * velika terca - moze (40)
 * kvarta       - neka (30)
 * triton       - fuj (-10)
 * kvinta       - moze (40)
 * mala seksta  - hmm... (20)
 * seksta       - woohoo (45)
 * mala septima - hmm... (20)
 * velika sept. - radije ne (1)
 * 
 */
const int NoteVectorEval::score[] = { 10, 5, 15, 50, 40, 30, -10, 35, 40, 45, 20, 1 };

